# tischgruppe Kompositum.py
# Fortsetzung der Entwicklung auf eine Tischgruppe
# mit den Methoden FuegeHinzu() und Entferne()
# und mit _AktualisiereAbmessungen() in Moebel

# Umsetzen des Kompositummusters

from grafikfenster import *

from moebel import Moebel
from tisch import Tisch
from stuhl import Stuhl

### ---------------------------------------------------
class Tischgruppe(Moebel):
    '''Die Klasse Tischgruppe erzeugt Objekte, die aus
    einem Tischobjekt und mehreren Stuhlobjekten bestehen koennen.
    Begonnen wird mit einer leeren Tischgruppe.
    Umsetzung des Kompositummusters in Transformiere von Moebel.'''
    def __init__(self,
                 xPos=50,
                 yPos=150,
                 winkel=0,
                 farbe="",
                 sichtbar=False):
        self.__moebel=[]
        Moebel.__init__(self, xPos, yPos, 0, 0, winkel, None, sichtbar)
        
    def GibFigur(self):
        """definiert die zu zeichnende Figur"""
        path = self.GibZeichenPfad()

        for moebel in self.__moebel:
            path.AddPath(moebel.GibFigur())

        return self.Transformiere(path)

    def FuegeHinzu(self, moebel):
        '''Die Sichtbarkeit des Moebel-Objekts wird
        auf die Sichtbarkeit der Gruppe gesetzt'''
        if moebel in self.__moebel: return
        self.__moebel.append(moebel)
        moebel.SetzeGruppe(self)
        self._AktualisiereAbmessungen()
        if self.GibSichtbar(): moebel.Zeige()
        else: moebel.Verberge()
        self.Update()
    
    def Entferne(self, moebel):
        '''Die Sichtbarkeit des Moebel-Objekts wird auf False gesetzt.
        Ob das sinnvoll ist, haengt von der konkreten Anwendung ab.'''
        if moebel not in self.__moebel: return
        moebel.Verberge()
        self.__moebel.remove(moebel)
        moebel.SetzeGruppe(None)
        self._AktualisiereAbmessungen()
        self.Update()
    
    def Update(self):
        """Update-Methode"""
        for moebel in self.__moebel:
            moebel.Update()

    ### Ergaenzung:
    def AendereFarbe(self, farbe):
        '''Nur begrenzt sinnvoll: Es wird eine gemeinsame Farbe
        fuer alle beteiligten Moebelobjekte erzwungen.'''
        super(Tischgruppe,self).AendereFarbe(farbe)
        for moebel in self.__moebel:
            moebel.AendereFarbe(farbe)
        

### ---------------------------------------------------
class TischgruppeApp(wx.App):
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        self.TestAnwendung()
        #self.fenster.ZeigeFillingFrame()
        return True

        
    def TestAnwendung(self):
        global tg,s1,s2,s3,s4,s5,s6,t
        print('global: tg,s1,s2,s3,s4,s5,s6,t')
        tg=Tischgruppe(sichtbar=True)
        t=Tisch(50,50)
        tg.FuegeHinzu(t)
        s1=Stuhl(0,60,winkel=270)
        tg.FuegeHinzu(s1)
        s2=Stuhl(60,0,winkel=0)
        tg.FuegeHinzu(s2)
        s3=Stuhl(120,0,winkel=0)
        tg.FuegeHinzu(s3)
        s4=Stuhl(180,60,winkel=90)
        tg.FuegeHinzu(s4)
        s5=Stuhl(60,120,winkel=180)
        tg.FuegeHinzu(s5)
        s6=Stuhl(120,120,winkel=180)
        tg.FuegeHinzu(s6)

    def TestEntfernen(self):
        global tg,s1
        tg.Entferne(s1)
        
    def TestWiederEinfuegen(self):
        global tg,s1
        tg.FuegeHinzu(s1)
        
### ---------------------------------------------------
if __name__ == '__main__':
    app = TischgruppeApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
